## Script for CT_inventorydata.xlsx
library(tidyverse)
library(readxl)
library(taxonomyCleanr)

# Load data 
#  Change data_path to suit local conditions
data_path <- '../InvsForClarke/'
data_file <- 'CT_inventorydata'
data_type <- '.xlsx'
origData <- read_excel(paste(data_path, data_file, data_type, sep=''))
	# CT: bunch of warnings here about problematic column names; is fine

# Reduce fields to iTree interests: Sci, Comm, DBH
	# CT: need GenSp column made from $Genus and $Species
iTreeData <- origData %>%
	mutate(GenSp = paste(Genus, Species, sep=' ')) %>%
	select(Town, SciName = GenSp, CommName = Common, DBH = Dbh)

# Do taxonomyCleanR 
#  Create the taxa map
dir.create('taxonCleanr')  # warning if already exists; is fine
map_path <- './taxonCleanr'
taxa_col <- 'SciName'
iTreeData <- as.data.frame(iTreeData)  # TCR rejects tibbles (so far)
taxa_map <- create_taxa_map(path = map_path, x = iTreeData, col = taxa_col)
#  The actual working taxa_map is the file taxa_map.csv; changes are
#  actually made there, then various functions reload this df from 
#  that file.  

#  Get unique taxa and counts
taxa_cts <- count_taxa(x = iTreeData, col = taxa_col, path = map_path)
#  CT: n=168

#  Trim excess characters from the taxa list -- 'spp.', e.g.
taxa_map <- trim_taxa(path = map_path)

# # Remove taxa [Included only for reference]
# output <- remove_taxa(path = map_path, input = 'Unsorted biomass')

# # Supported authorities are listed in the column titled resolve_sci_taxa
# view_taxa_authorities()

#  Resolve scientific taxa using ITIS: 3, COL: 1, GBIF: 11
#		Pass 1 of 3 -- first pass fixes a lot of typo-type errors
taxa_map <- resolve_sci_taxa(path = map_path, data.sources = c(3,11))
#taxa_map <- resolve_sci_taxa(path = map_path, data.sources = c(11))

# # Resolve common (ITIS only)
# # => pretty useless
# outputC <- resolve_comm_taxa(path = map_path, data.sources = 3)

#  See unresolved entries
taxa_map[is.na(taxa_map$taxa_clean), c(1,5)]

#  Correct unresolved entries, as possible
	# CT: unresolved by ITIS & GBIF:
repl_list <- list('Evodia danielli' = 'Evodia daniellii',
									'Fagus silvatica' = 'Fagus sylvatica',
									'Ulmus thomasi' = 'Ulmus thomasii')
map2(names(repl_list), repl_list, function(bad, good) {
		replace_taxa(path = map_path, input = bad, output = good)
		return()})

#  Get changed taxa
changes <- taxa_map[(taxa_map$taxa_raw != taxa_map$taxa_clean)
										& (is.na(taxa_map$taxa_trimmed)), c(1:3,5)]

#  Resolve scientific taxa using ITIS: 3, COL: 1, GBIF: 11
#		Pass 2 of 3 -- second pass incorporates the unresolved
taxa_map <- resolve_sci_taxa(path = map_path, data.sources = c(3,11))
#  See unresolved entries
taxa_map[is.na(taxa_map$taxa_clean), c(1,5)]


# Get Eco PLANTS codes
spLsFile <- '../EcoSpeciesList_20July2018.csv'
SpecList <- read_csv(spLsFile, skip=1,
								 col_names=c('ID','Code','SName','CName','Genus','Host'))
eco_matches <- left_join(taxa_map, SpecList,
											by=c('taxa_clean' = 'SName'))

#  Check for records missing codes
table(eco_matches[is.na(eco_matches$Code),]$taxa_raw)

	# CT: unmatched on Eco list
repl_list <- list('Aesculus octandra' = 'Aesculus flava',
									'Carya tomentosa' = 'Carya alba',
									'Cladrastis lutea' = 'Cladrastis kentukea',
									'Evodia danielli' = 'Tetradium daniellii',
									'Gymnocladus dioicus' = 'Gymnocladus dioicus',
									'Oxydendron spp' = 'Oxydendrum',
									'Phanus frangula' = 'Frangula alnus',
									'Picea nigra' = 'Picea mariana',
									'Platanus acerifolia' = 'Platanus x acerifolia',
# these are false species names caught by inspection
									'Populus balsamea' = 'Populus balsamifera',
									'Ulmus rubrum' = 'Ulmus rubra',
									'Pinus flexius' = 'Pinus flexilis')
map2(names(repl_list), repl_list, function(bad, good) {
		replace_taxa(path = map_path, input = bad, output = good)
		return()})

# [non-correct move to reload taxa_map]
taxa_map <- trim_taxa(path = map_path)

#  Resolve scientific taxa using ITIS: 3, COL: 1, GBIF: 11
#		Pass 3 of 3 -- third pass incorporates the codeless fixes
taxa_map <- resolve_sci_taxa(path = map_path, data.sources = c(3,11))

#  Get unresolved entries -- should be none
taxa_map[is.na(taxa_map$taxa_clean), c(1,5)]

#  Check again for missing codes -- should be none
eco_matches <- left_join(taxa_map, SpecList,
											by=c('taxa_clean' = 'SName'))
table(eco_matches[is.na(eco_matches$Code),]$taxa_raw)

# CT: Platanus acerifolia should be Platanus x acerifolia
# => hand-edit taxa_clean in taxa_map.csv...??

#  Get changed taxa; write csv of changed names
changes <- taxa_map[(taxa_map$taxa_raw != taxa_map$taxa_clean)
										& (is.na(taxa_map$taxa_trimmed)), c(1:3,5)]
write_csv(changes, paste0(map_path, '/taxa_changed.csv'))


#  Revise the raw data table and write to file
taxa_output <- revise_taxa(path = map_path, x = iTreeData, col = taxa_col, sep = '\t')

# Reduce to what iTree Eco wants
taxa_output <- taxa_output[1:5]

taxa_output <- left_join(taxa_output, SpecList,
											by=c('taxa_clean' = 'SName'))[-c(6,8:10)]

# Write csv
#outfile = ''
#write_csv(taxa_output, paste0(map_path, '/', outfile))

# Or: To split one multi-town file into dfs by town
statePref = 'CT_'
dataList <- group_split(taxa_output, Town, keep=FALSE) %>%
	setNames(unique(taxa_output$Town))
mapply(write_csv, dataList, path = paste0(map_path, '/', 
									statePref, names(dataList), '_inventory.csv'))
