## Script for Yarmouth_Maine_Sept_2012.xlsx
# => Has only common names, so have to translate with Commons Dictionary
library(tidyverse)
library(readxl)
library(taxonomyCleanr)

# Function for case-insensitive joins, nec. for matching CommNames; 
# from https://gist.github.com/jimhester/a060323a05b40c6ada34
# usage: 
# insensitive(left_join)(a, b,  
#    by = list(x = c('letter', 'num'), y = c('letter', 'num2')))
insensitive <- function(fun = inner_join) {
	new_fun <- fun
	body(new_fun) <- substitute({
		by <- dplyr:::common_by(by, x, y)
		tmp_by_x <- paste0("_", by$x, "_")
		tmp_by_y <- paste0("_", by$y, "_")
		for (i in seq_along(by$x)) {
			x[[tmp_by_x[[i]]]] <- tolower(x[[by$x[[i]]]])
			y[[tmp_by_y[[i]]]] <- tolower(y[[by$y[[i]]]])
			y[[by$y[[i]]]] <- NULL
		}
		res <- fun(x, y, list(x = tmp_by_x, y = tmp_by_y))
		res[tmp_by_x] <- list(NULL)
		res
	})
	new_fun
}


# Load data 
#  Change data_path to suit local conditions
data_path <- '../InvsForClarke/'
data_file <- 'Yarmouth_Maine_Sept_2012'
data_type <- '.xlsx'
town_name <- 'ME_Yarmouth'
origData <- read_excel(paste0(data_path, data_file, data_type),
											 sheet='Yarmouth_Maine_Sept_2012')

# Reduce fields to iTree interests: Sci, Comm, DBH
	# Kennebunkport: Only has common names
iTreeData <- origData %>%
	mutate(SciName = NA) %>%
	select(SciName, CommName = Species, DBH = DBH)
iTreeData[iTreeData$CommName %in% 'PRSIMEN', 2] <- 'PERSIMMON'

## Match common names to SciNames in commons dictionary
#  If commsFile doesn't exist run commDict.R
commsFile <- 'commonsDict.csv'
commsDict <- read_csv(commsFile)

comm_matches <- insensitive(left_join)(iTreeData, commsDict,
											by=c('CommName'))
# comm_matches <- left_join(iTreeData, commsDict,
# 											by=c('CommName'))
nonComms <- comm_matches[is.na(comm_matches$SciName.y),]
table(nonComms$CommName)

iTreeData <- comm_matches %>% 
	select(SciName = SciName.y, CommName, DBH)


# Do taxonomyCleanR 
#  Create the taxa map
dir.create(paste0(town_name, '/taxonCleanr'), recursive=TRUE)  # warning if already exists; is fine
out_path <- paste0('./', town_name)
map_path <- paste0('./', town_name, '/taxonCleanr')
taxa_col <- 'SciName'
iTreeData <- as.data.frame(iTreeData)  # TCR rejects tibbles (so far)
taxa_map <- create_taxa_map(path = map_path, x = iTreeData, col = taxa_col)
#  The actual working taxa_map is the file taxa_map.csv; changes are
#  actually made there, then various functions reload this df from 
#  that file.  

#  Get unique taxa and counts
taxa_cts <- count_taxa(x = iTreeData, col = taxa_col, path = map_path)
#  Kennebunkport: n=47

#  Trim excess characters from the taxa list -- 'spp.', e.g.
taxa_map <- trim_taxa(path = map_path)

# # Remove taxa [Included only for reference]
# output <- remove_taxa(path = map_path, input = 'Unsorted biomass')

# # Supported authorities are listed in the column titled resolve_sci_taxa
# view_taxa_authorities()

#  Resolve scientific taxa using ITIS: 3, COL: 1, GBIF: 11
#		Pass 1 of 1 -- first pass fixes a lot of typo-type errors
taxa_map <- resolve_sci_taxa(path = map_path, data.sources = c(3))

# Resolve common (ITIS only)
# => pretty useless
# taxa_map <- resolve_comm_taxa(path = map_path, data.sources = 3)

#  See unresolved entries
taxa_map[is.na(taxa_map$taxa_clean), c(1,5)]

#  Correct unresolved entries, as possible
	# Yarmouth: None!
# repl_list <- list('Cercidiphyllum japonicum' = 'Cercidiphyllum japonicum')
# map2(names(repl_list), repl_list, function(bad, good) {
# 		replace_taxa(path = map_path, input = bad, output = good)
# 		return()})

# [non-correct move to reload taxa_map]
taxa_map <- trim_taxa(path = map_path)

#  Get changed taxa
changes <- taxa_map[(taxa_map$taxa_raw != taxa_map$taxa_clean)
										& (is.na(taxa_map$taxa_trimmed)), c(1:3,5)]


# Get Eco PLANTS codes
spLsFile <- '../EcoSpeciesList_20July2018.csv'
SpecList <- read_csv(spLsFile, skip=1,
								 col_names=c('ID','Code','SName','CName','Genus','Host'))
eco_matches <- left_join(taxa_map, SpecList,
											by=c('taxa_clean' = 'SName'))
											# by=c('taxa_raw' = 'CName'))

#  Check for records missing codes
table(eco_matches[is.na(eco_matches$Code),]$taxa_raw)

	# Yarmouth: none!
# repl_list <- list('Platanus x acerifolia' = 'Platanus x acerifolia')
# map2(names(repl_list), repl_list, function(bad, good) {
# 		replace_taxa(path = map_path, input = bad, output = good)
# 		return()})

# [non-correct move to reload taxa_map]
taxa_map <- trim_taxa(path = map_path)

# Make modified taxa_map2 df forcing Eco names into $taxa_clean 
#  => *Does Not Match File*
# Yarmouth: no changes; skip it
# taxa_map2 <- taxa_map %>% mutate(taxa_clean = 
# 													 	if_else(is.na(taxa_replacement), taxa_clean, taxa_replacement))

#  Check again for missing codes (against taxa_map2) -- should be none
eco_matches <- left_join(taxa_map, SpecList,
											by=c('taxa_clean' = 'SName'))
table(eco_matches[is.na(eco_matches$Code),]$taxa_raw)

#  Get changed taxa; write csv of changed names
changes <- taxa_map[(! taxa_map$taxa_raw %in% taxa_map$taxa_clean)
										& (is.na(taxa_map$taxa_trimmed)), c(1:3,5)]
write_csv(changes, paste0(map_path, '/taxa_changed.csv'))

# => Hackish: overwrite taxa_map.csv with taxa_map2
# write_csv(taxa_map2, paste0(map_path, '/taxa_map.csv'))

#  Revise the raw data table and write to file
taxa_output <- revise_taxa(path = map_path, x = iTreeData, col = taxa_col, sep = '\t')

# Reduce to what iTree Eco wants
taxa_output <- taxa_output[1:5]

# Stick on PLANTS codes
taxa_output <- left_join(taxa_output, SpecList,
											by=c('taxa_clean' = 'SName'))[-c(6,8:10)]

table(taxa_output$SciName[is.na(taxa_output$Code)])
table(taxa_output$taxa_clean[is.na(taxa_output$Code)])

# Write csv
outfile = paste0(town_name, '_inventory.csv')
write_csv(taxa_output, paste0(out_path, '/', outfile))


# Or: To split one multi-town file into dfs by town
# statePref = 'CT_'
# dataList <- group_split(taxa_output, Town, keep=FALSE) %>%
# 	setNames(unique(taxa_output$Town))
# mapply(write_csv, dataList, path = paste0(map_path, '/', 
# 									statePref, names(dataList), '_inventory.csv'))

