## Script for Syracuse_myTreeKeeper_5-9-2018.xlsx
library(tidyverse)
library(readxl)
library(taxonomyCleanr)

# Load data 
#  Change data_path to suit local conditions
data_path <- '../InvsForClarke/'
data_file <- 'Syracuse_myTreeKeeper_5-9-2018'
data_type <- '.xlsx'
origData <- read_excel(paste0(data_path, data_file, data_type))

# Reduce fields to iTree interests: Sci, Comm, DBH
#  mutate: strip all (most) cultivars
iTreeData <- origData %>%
	mutate(SciName = str_replace(Sci_Name, " *[\'\"].+$", "")) %>%
	select(SciName, CommName = Common_Name, DBH)

# Do taxonomyCleanR 
#  Create the taxa map
dir.create('NY_Syracuse/taxonCleanr', recursive=TRUE)  # warning if already exists; is fine
map_path <- './NY_Syracuse/taxonCleanr'
taxa_col <- 'SciName'
iTreeData <- as.data.frame(iTreeData)  # TCR rejects tibbles (so far)
taxa_map <- create_taxa_map(path = map_path, x = iTreeData, col = taxa_col)
#  The actual working taxa_map is the file taxa_map.csv; changes get
#  made there, then various functions reload this df from that file.  

#  Get unique taxa and counts
taxa_cts <- count_taxa(x = iTreeData, col = taxa_col, path = map_path)
#  Springfield: n=238

#  Trim excess characters from the taxa list -- 'spp.', e.g.
taxa_map <- trim_taxa(path = map_path)

# # Remove taxa [Included only for reference]
# output <- remove_taxa(path = map_path, input = 'Unsorted biomass')

# # Supported authorities are listed in the column titled resolve_sci_taxa
# view_taxa_authorities()

#  Resolve scientific taxa using ITIS: 3, COL: 1, GBIF: 11
#		Pass 1 of 2 -- first pass fixes a lot of typo-type errors
#taxa_map <- resolve_sci_taxa(path = map_path, data.sources = c(3,11))
taxa_map <- resolve_sci_taxa(path = map_path, data.sources = c(11))

#  See unresolved entries
taxa_map[is.na(taxa_map$taxa_clean), c(1,5)]

#  Correct unresolved entries, as possible
	# Syracuse: unresolved by GBIF:
repl_list <- list('Gymnocladus dioicus' = 'Gymnocladus dioicus',
									'Crataegus crusgalli' = 'Crataegus crus-galli',
									# These pulled from changes df, below
									'arbor (Thuja occidentalis' = 'Thuja occidentalis',
									'Platanus x acerifolia' = 'Platanus x acerifolia',
									'Prunus x yedoensis' = 'Prunus x yedoensis',
									'Acer x freemanii' = 'Acer x freemanii' ) # ,
									# 'Magnolia x Soulangiana' = 'Magnolia x soulangiana',
									# "Prunus, cerasifera 'Newportii'" = 'Prunus cerasifera')
map2(names(repl_list), repl_list, function(bad, good) {
		replace_taxa(path = map_path, input = bad, output = good)
		return()})
# Should throw a list of NULLs

#  Get changed taxa
# changes <- taxa_map[(taxa_map$taxa_raw != taxa_map$taxa_clean)
changes <- taxa_map[(! taxa_map$taxa_raw %in% taxa_map$taxa_clean)
										& (is.na(taxa_map$taxa_trimmed)), c(1:3,5)]

#  Resolve scientific taxa using ITIS: 3, COL: 1, GBIF: 11
#		Pass 2 of 3 -- second pass incorporates the previously unresolved
taxa_map <- resolve_sci_taxa(path = map_path, data.sources = c(11))
#  See unresolved entries #2
taxa_map[is.na(taxa_map$taxa_clean), c(1,5)]
# Should be no more species listed; junk/comment entries are okay

# Get Eco PLANTS codes
spLsFile <- '../EcoSpeciesList_20July2018.csv'
SpecList <- read_csv(spLsFile, skip=1,
								 col_names=c('ID','Code','SName','CName','Genus','Host'))
eco_matches <- left_join(taxa_map, SpecList,
											by=c('taxa_clean' = 'SName'))

#  Check for records that didn't find codes
table(eco_matches[is.na(eco_matches$Code),]$taxa_raw)

## Testing ways to deal with cultivars; not there yet
### Trying fuzzyjoin
# install.packages('fuzzyjoin')
# library('fuzzyjoin')
# fzzy_map <- mutate(taxa_map, taxa_fzzy = taxa_clean) 
# fzzy_map[is.na(fzzy_map$taxa_fzzy),11] = '000'
# fzzy_matches <- stringdist_left_join(fzzy_map, SpecList, 
# 											by=c('taxa_fzzy' = 'SName'),
# 											distance_col='dist')

	# Springfield: unmatched on Eco list
	#   Some of these are identities, to reset names that get changed
	#   by TCR authorities
repl_list <- list('Acer x freemanii' = 'Acer x freemanii',
									'Acer freemanii' = 'Acer x freemanii',
									'Acer tataricum ginnala' = 'Acer tataricum',
									'Amelanchier grandifolia' = 'Amelanchier x grandiflora',
									'Carya illinoensis' = 'Carya illinoinensis',
									'Carya tomentosa' = 'Carya alba',
									'Cladrastis lutea' = 'Cladrastis kentukea',
									# 'Fraxinus lanceolata Patmore' = 'Fraxinus pennsylvanica',
									'Fraxinus lanceolata' = 'Fraxinus pennsylvanica',
									'Gleditsia triacanthos inermis' = 'Gleditsia triacanthos',
									'Gymnocladus dioicus' = 'Gymnocladus dioicus',
									'Kalopanax pictus' = 'Kalopanax septemlobus',
									'Platanus x acerifolia' = 'Platanus x acerifolia',
									'Prunus x yedoensis' = 'Prunus yedoensis', # Eco has only cultivar
									'Rhamnus frangula' = "Frangula alnus", 
									'Rhus typhina' = 'Rhus hirta', # Eco is behind here
									'Styphnolobium japonicum' = 'Sophora japonica') # Eco is behind here
map2(names(repl_list), repl_list, function(bad, good) {
		replace_taxa(path = map_path, input = bad, output = good)
		return()})
# Should throw a list of NULLs

# [non-correct move to reload taxa_map]
taxa_map <- trim_taxa(path = map_path)

# Make modified taxa_map2 df forcing Eco names into $taxa_clean 
#  => *Does Not Match File*
taxa_map2 <- taxa_map %>% mutate(taxa_clean = 
							 	if_else(is.na(taxa_replacement), taxa_clean, taxa_replacement))

#  Check again for missing codes (against taxa_map2)
eco_matches <- left_join(taxa_map2, SpecList,
											by=c('taxa_clean' = 'SName'))
table(eco_matches[is.na(eco_matches$Code),]$taxa_raw)
 # Should be no species; possibly junk categories

#  Get changed taxa; write csv of changed names
changes <- taxa_map2[(! taxa_map2$taxa_raw %in% taxa_map2$taxa_clean)
										& (is.na(taxa_map2$taxa_trimmed)), c(1:3,5)]
write_csv(changes, paste0(map_path, '/taxa_changed.csv'))

# => Hackish: overwrite taxa_map.csv with taxa_map2
write_csv(taxa_map2, paste0(map_path, '/taxa_map.csv'))

#  Revise the raw data table and write to file
taxa_output <- revise_taxa(path = map_path, x = iTreeData, col = 'SciName', sep = '\t')

# Reduce to what iTree Eco wants
taxa_output <- taxa_output[1:5]

# Stick on PLANTS codes
taxa_output <- left_join(taxa_output, SpecList,
											by=c('taxa_clean' = 'SName'))[-c(6,8:10)]

table(taxa_output$SciName[is.na(taxa_output$Code)])
table(taxa_output$taxa_clean[is.na(taxa_output$Code)])

# Write csv
outfile = 'NY_Syracuse_inventory.csv'
write_csv(taxa_output, paste0(map_path, '/', outfile))

# # Or: To split one multi-town file into dfs by town
# statePref = 'CT_'
# dataList <- group_split(taxa_output, Town, keep=FALSE) %>%
# 	setNames(unique(taxa_output$Town))
# mapply(write_csv, dataList, path = paste0(map_path, '/', 
# 									statePref, names(dataList), '_inventory.csv'))
